; this file was produced from Oracle 8.0.4 OCI.DLL by pexports program
; written by Anders Norlander <anorland@hem2.passagen.se>
; http://www.acc.umu.se/~anorland/gnu-win32/index.html
;
; to make liboci.a run
; dlltool --input-def oci.def --output-lib liboci.a
;
LIBRARY OCI.dll
EXPORTS
OCILobGetChunkSize
OCINlsCharSetIdToName
OCIAQDeq
OCIAQEnq
OCIAttrGet
OCIAttrSet
OCIBindArrayOfStruct
OCIBindByName
OCIBindByPos
OCIBindDynamic
OCIBindObject
OCIBreak
OCICacheFlush
OCICacheFlushRefresh
OCICacheFree
OCICacheGetObjects
OCICacheRefresh
OCICacheRegister
OCICacheUnmark
OCICacheUnpin
OCICollAppend
OCICollAssign
OCICollAssignElem
OCICollGetElem
OCICollMax
OCICollSize
OCICollTrim
OCIDBStartup
OCIDBShutdown
OCIDateAddDays
OCIDateAddMonths
OCIDateAssign
OCIDateCheck
OCIDateCompare
OCIDateDaysBetween
OCIDateFromText
OCIDateLastDay
OCIDateNextDay
OCIDateSysDate
OCIDateToText
OCIDateZoneToZone
OCIDefineArrayOfStruct
OCIDefineByPos
OCIDefineDynamic
OCIDefineObject
OCIDescribeAny
OCIDescriptorAlloc
OCIDescriptorFree
OCIDurationBegin
OCIDurationEnd
OCIDurationGetParent
OCIEnvInit
OCIErrorGet
OCIHandleAlloc
OCIHandleFree
OCIInitialize
OCIIterCreate
OCIIterDelete
OCIIterGetCurrent
OCIIterInit
OCIIterNext
OCIIterPrev
OCILdaToSvcCtx
OCILobAppend
OCILobAssign
OCILobCharSetForm
OCILobCharSetId
OCILobCopy
OCILobDisableBuffering
OCILobEnableBuffering
OCILobErase
OCILobFileClose
OCILobFileCloseAll
OCILobFileExists
OCILobFileGetName
OCILobFileIsOpen
OCILobFileOpen
OCILobFileSetName
OCILobFlushBuffer
OCILobGetLength
OCILobIsEqual
OCILobLoadFromFile
OCILobLocatorIsInit
OCILobRead
OCILobTrim
OCILobWrite
OCILobWriteAppend
OCILogoff
OCILogon
OCINumberAbs
OCINumberAdd
OCINumberArcCos
OCINumberArcSin
OCINumberArcTan
OCINumberArcTan2
OCINumberAssign
OCINumberCeil
OCINumberCmp
OCINumberCos
OCINumberDiv
OCINumberExp
OCINumberFloor
OCINumberFromInt
OCINumberFromReal
OCINumberFromText
OCINumberHypCos
OCINumberHypSin
OCINumberHypTan
OCINumberIntPower
OCINumberIsZero
OCINumberLn
OCINumberLog
OCINumberMod
OCINumberMul
OCINumberNeg
OCINumberPower
OCINumberRound
OCINumberSetZero
OCINumberSin
OCINumberSqrt
OCINumberSub
OCINumberTan
OCINumberToInt
OCINumberToReal
OCINumberToText
OCINumberTrunc
OCIObjectAlwaysLatest
OCIObjectArrayPin
OCIObjectCopy
OCIObjectExists
OCIObjectFlush
OCIObjectFlushRefresh
OCIObjectFree
OCIObjectGetAttr
OCIObjectGetInd
OCIObjectGetObjectRef
OCIObjectGetProperty
OCIObjectGetTypeRef
OCIObjectIsDirtied
OCIObjectIsDirty
OCIObjectIsLoaded
OCIObjectIsLocked
OCIObjectLock
OCIObjectMarkDelete
OCIObjectMarkDeleteByRef
OCIObjectMarkUpdate
OCIObjectNew
OCIObjectNotAlwaysLatest
OCIObjectPin
OCIObjectPinCountReset
OCIObjectPinTable
OCIObjectRefresh
OCIObjectSetAttr
OCIObjectUnmark
OCIObjectUnmarkByRef
OCIObjectUnpin
OCIParamGet
OCIParamSet
OCIPasswordChange
OCIRawAllocSize
OCIRawAssignBytes
OCIRawAssignRaw
OCIRawPtr
OCIRawResize
OCIRawSize
OCIRefAssign
OCIRefClear
OCIRefFromHex
OCIRefHexSize
OCIRefIsEqual
OCIRefIsNull
OCIRefToHex
OCIResultSetToStmt
OCISecurityAbortIdentity
OCISecurityClosePersona
OCISecurityCloseWallet
OCISecurityCreateIdentity
OCISecurityCreatePersona
OCISecurityCreateWallet
OCISecurityDeEnvelope
OCISecurityDecrypt
OCISecurityDestroyWallet
OCISecurityEncrypt
OCISecurityEncryptExpansion
OCISecurityEnvelope
OCISecurityFreeIdentity
OCISecurityGetIdentity
OCISecurityGetProtection
OCISecurityHash
OCISecurityHashExpansion
OCISecurityInitBlock
OCISecurityInitialize
OCISecurityKeyedHash
OCISecurityKeyedHashExpansion
OCISecurityOpenPersona
OCISecurityOpenWallet
OCISecurityPKDecrypt
OCISecurityPKEncryptExpansion
OCISecurityPurgeBlock
OCISecurityRandomBytes
OCISecurityRandomNumber
OCISecurityRemoveIdentity
OCISecurityRemovePersona
OCISecurityReuseBlock
OCISecuritySeedRandom
OCISecuritySetBlock
OCISecuritySetProtection
OCISecuritySign
OCISecuritySignDetExpansion
OCISecuritySignDetached
OCISecuritySignExpansion
OCISecurityStorePersona
OCISecurityStoreTrustedIdentity
OCISecurityTerminate
OCISecurityValidate
OCISecurityVerify
OCISecurityVerifyDetached
OCISecurity_PKEncrypt
OCIServerAttach
OCIServerDetach
OCIServerVersion
OCIServerRelease
OCISessionBegin
OCISessionEnd
OCIStmtBindByName
OCIStmtBindByPos
OCIStmtExecute
OCIStmtFetch
OCIStmtFetch2
OCIStmtGetBindInfo
OCIStmtGetPieceInfo
OCIStmtPrepare
OCIStmtSetPieceInfo
OCIStringAllocSize
OCIStringAssign
OCIStringAssignText
OCIStringPtr
OCIStringResize
OCIStringSize
OCISvcCtxToLda
OCITableDelete
OCITableExists
OCITableFirst
OCITableLast
OCITableNext
OCITablePrev
OCITableSize
OCITransCommit
OCITransDetach
OCITransForget
OCITransPrepare
OCITransRollback
OCITransStart
OCITypeArrayByName
OCITypeArrayByRef
OCITypeAttrByName
OCITypeAttrNext
OCITypeAttrs
OCITypeByName
OCITypeByRef
OCITypeCollElem
OCITypeCollExtTypeCode
OCITypeCollSize
OCITypeCollTypeCode
OCITypeElemCharSetForm
OCITypeElemCharSetID
OCITypeElemDefaultValue
OCITypeElemExtTypeCode
OCITypeElemFlags
OCITypeElemLength
OCITypeElemName
OCITypeElemNumPrec
OCITypeElemNumScale
OCITypeElemParamMode
OCITypeElemParameterizedType
OCITypeElemType
OCITypeElemTypeCode
OCITypeIterFree
OCITypeIterNew
OCITypeIterSet
OCITypeMethodByName
OCITypeMethodEncap
OCITypeMethodFlags
OCITypeMethodMap
OCITypeMethodName
OCITypeMethodNext
OCITypeMethodOrder
OCITypeMethodOverload
OCITypeMethodParams
OCITypeMethods
OCITypeName
OCITypeParamByName
OCITypeParamByPos
OCITypeParamPos
OCITypeResult
OCITypeSchema
OCITypeTypeCode
OCITypeVTInit
OCITypeVTInsert
OCITypeVTSelect
OCITypeVersion
obindps
obndra
obndrn
obndrv
obreak
ocan
ocibre
ocibrn
ocibrv
ociclo
ocicof
ocicom
ocidefn
ocidfn
ocidsc
ociepacm
ocieperr
ociepgoe
ociepmsg
ocierr
ociexe
ociexn
ocifet
ocilof
ocilon
ociope
ocirlo
ocirol
ocisq3
oclose
ocof
ocom
ocon
odefin
odefinps
odescr
odessp
odsc
oerhms
oermsg
oexec
oexfet
oexn
ofen
ofetch
oflng
ogetpi
ognfd
olog
ologof
ologon
olon
oname
onbclr
onbset
onbtst
oopen
oopt
oparse
opinit
orlon
orol
ortgcty
osetpi
osql3
OCINlsEnvironmentVariableGet
OCIEnvNlsCreate
OCINlsCharSetNameToId
OCILobIsTemporary
OCILobFreeTemporary
OCILobLocatorAssign
OCILobCreateTemporary
OCIXMLTypeCreateFromSrc
OCIDateTimeToText
OCIIntervalToText
OCIDateTimeGetTimeZoneOffset
OCIPing
OCISessionRelease
OCISessionPoolDestroy
OCISessionPoolCreate
OCISessionGet
OCISessionPoolDestroy

